@@ -22,34 +22,6 @@ sa <- select_predictors(sa, c("wc2.1_10m_bio_1","wc2.1_10m_bio_12"))
 sa <- set_predictor_names(sa, c("bio1", "bio12"))
 scen <- scen[,,,c("bio1", "bio12")]
 
-# Invasiveness tests
-#library(stars)
-#library(sf)
-#sf_use_s2(FALSE)
-#scen2 <- read_stars("~/Documents/caretSDM_bkup/input_data/WorldClim_data_future/mi_ssp585_10_2090.tif")
-#current <- read_stars(list.files("~/Documents/caretSDM_bkup/input_data/WorldClim_data_current/", full.names=T), along = "band")
-#names(current) <- "current"
-#current <- sf::st_crop(current[,,,c("1.tif", "4.tif", "12.tif")], rs)
-#current <- stars::st_set_dimensions(current, "band", values = c("bio1", "bio4","bio12"))
-#
-#future <- read_stars(list.files("~/Documents/Mapas/Rasters/WorldClim 2.1/future_10m/", full.names=T))
-#names(future) <- names(scen)
-#future <- sf::st_crop(future[,,,c("bio01", "bio04", "bio12")], rs)
-#future <- stars::st_set_dimensions(future, "band", values = c("bio1", "bio4","bio12"))
-#
-#br <- st_read("~/Documents/caretSDM_bkup/input_data/Brasil/estadosl_2007.shp")
-#rs <- br[br$SIGLAUF3 == "RS",]
-#st_crs(rs) <- 4326
-#scen2 <- sf::st_crop(scen2[,,,c("bio01", "bio04", "bio12")], rs)
-#scen2 <- stars::st_set_dimensions(scen2, "band", values = c("bio1", "bio4","bio12"))
-#sf_use_s2(TRUE)
-#scen <- scen2
-#scenarios_names = NULL
-#pred_as_scen = FALSE
-#variables_selected = NULL
-#stationary = NULL
-
-
 test_that("add_scenarios - invasiveness", {
   sa_pred <- add_scenarios(sa, scen_rs[,,,c("bio1", "bio12")], pred_as_scen = FALSE)
   expect_equal(
@@ -105,7 +77,7 @@ test_that("add_scenarios - stars selecionando uma variável", {
 })
 
 test_that("add_scenarios - NULL", {
-  sa_pred <- add_scenarios(sa, variables_selected=c("bio1"))
+  sa_pred <- add_scenarios(sa, variables_selected = c("bio1"))
   expect_equal(
     sa_pred$scenarios$data$current,
     sa_pred$grid
@@ -344,3 +316,24 @@ test_that("add_scenarios", {
   expect_equal(class(scenarios$a), c("sf", "data.frame"))
   expect_null(get_scenarios_data(occ))
 })
+
+test_that("add_scenarios - crop_area with different crs", {
+  bioc <- bioc[,,,c(1,3)]
+  parana2 <- sf::st_transform(parana, 6933)
+  expect_true(sf::st_crs(parana2) != sf::st_crs(scen))
+  sa_pred <- add_scenarios(sa, scen, crop_area = parana2)
+  #sa_pred$scenarios$data$current |> select("cell_id") |> plot()
+  #sa_pred$scenarios$data$ca_ssp245_2090 |> select("cell_id") |> plot()
+
+
+  # Testar bbox dos cenários futuros e presente.
+  expect_equal(nrow(sa_pred$scenarios$data$ca_ssp245_2090),
+               nrow(sa_pred$scenarios$data$current))
+  expect_equal(
+    scenarios_names(sa_pred),
+    c("ca_ssp245_2090", "ca_ssp585_2090", "mi_ssp245_2090", "mi_ssp585_2090", "current")
+  )
+  expect_true(sum(na.omit(as.data.frame(bioc))$band == "bio1") > sa_pred$scenarios$data$current |> nrow())
+  expect_true(sum(na.omit(as.data.frame(scen["ca_ssp245_2090"]))$band == "bio1") >
+                sa_pred$scenarios$data$ca_ssp245_2090 |> nrow())
+})
