@@ -70,7 +70,8 @@ test_that("data_clean - normal path with pred at wgs84", {
                      coords = c("decimalLongitude", "decimalLatitude"),
                      crs = 6933)
   occ2 <- sf::st_transform(occ2, crs = 4326)
-  suppressWarnings(oc <- occurrences_sdm(occ2, independent_test = TRUE, crs= 4326) |> join_area(pred))
+  oc <- occurrences_sdm(occ2, independent_test = TRUE, crs= 4326) |>
+    join_area(pred)
   i <- input_sdm(oc, pred)
   i <- data_clean(i, terrestrial=FALSE)
   expect_true(sf::st_crs(i$occurrences$occurrences) == sf::st_crs(oc$occurrences))
